#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2020 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>>Custom Font Tools>>Polish vx.xx...'
#
# Change Log
# Version 0.01 Provision added to call Shine Filter v 0.01 or v 0.01b by Graechan and Chisel or Carve Filter version 1.3 by Rob Antonishen
# Version 0.02 Outline width set to zero will allow the filter to add shine and/or chisel to existing custom font layers without adding outlines when outline width set to zero
#              Distort will also work on all these outlines apart from the brushed outline when outline width set to zero and Distort selection spread value > 0
# Version 0.03 Trimmed pixels from letter faces to remove slight halo effect
# Version 0.04 Added gaussian blur after chisel effect to smooth artifacts using pdb.plug_in_sel_blur with options for blur radius and blur delta
# Version 0.05 Removed dependency on shine and chisel plug-ins and replaced calls with embedded code
#              Added option to colour the shiny gloss

# Current Version of Custom Font Outlines 0.05
# Version 0.01 Effects Only
#              Separated from Outlines to leave only Shine and Chisel or Carve effects to reduce dialog height.
# Version 0.02 Effects Only
#              Added Emboss and Bumpmap Options
# Version 0.03 Effects Only
#              Added separate options for effects
# Version 0.04 Effects Only
#              Added trim option with shine effect for user to remove pixels from the character borders
# Version 0.05 Effects Only
#              Adjusted settings for bevel effect (bevel grey used and mode opacity increased to full)
# Version 0.06 Effects Only
#              Added option to offset shine; fixed bevel depth dialog setting error
# Version 0.07 Effects Only
#              Tweaked emboss - removed alpha selection during first gaussian blur and embossing before re-applying selection
# Version 0.08 Effects Only
#              Updated bumpmap
# Version 0.09 Effects Only
#              Fixed error in use of selective gaussian blur plug-in in bevel option, accepts unicode symbol characters 
# Migrated to
# Sheen N Shine
# Version 0.01 Offers two effects i.e. the existing Shine effect and new Sheen effect - a sort of reflected gloss
#			   Sheen can be used with Shine or separately takes colour from currently set bg colour
# Version 0.02 Increased range of modes and density of sheen 
# Version 0.03 Added slider to adjust emboss darkness
# Version 0.04 Maintain shine with higher emboss widths 

from gimpfu import *
import math

#midgrey = gimpcolor.RGB(100, 100, 100)

#def do_stuff(image, layer, use_texture, bump_depth, bump_waterlevel, bump_ambient, use_shine, use_sheen, emboss_width, shine_offset, trim_value, gloss_colour, add_sheen, sheen_opacity, blur_radius, blur_delta, effect_elevation, effect_azimuth, effect_mode_ptr):
def do_stuff(image, layer, use_shine, use_sheen, emboss_width, highlight_blur, shine_offset, trim_value, emboss_amount, gloss_colour, add_sheen, sheen_opacity, blur_radius, blur_delta, effect_elevation, effect_azimuth, effect_mode_ptr):

    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
    #use_clipboard_as_brush, do_layer_info, do_layer_names, do_path_names, do_sort = 0,0,0,0,0
    outline_colour = pdb.gimp_context_get_foreground()
	
    # Define some colours for proc chisel
    black = gimpcolor.RGB(0, 0, 0)
    white = gimpcolor.RGB(255, 255, 255)
    darkgrey = gimpcolor.RGB(50, 50, 50)
    midgrey = gimpcolor.RGB(125, 125, 125)
    grey_value = int(150 - emboss_amount)
    grey = gimpcolor.RGB(grey_value, grey_value, grey_value)
    polish_colour = pdb.gimp_context_get_background()
	
    # set non UI shine plugin parameters
    shine_shadow_X, shine_shadow_Y, shine_shadow_size, shine_shadow_opacity, shine_keep_selection, shine_keep_layers = 0,0,0,0,0,0
    use_texture = False
	
    if use_sheen: use_shine = False
	
    # set non UI chisel plugin parameters
    chisel_depth, chisel_location, chisel_keep_bump = 60, 0, 0

    loop_ptr = 1
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    layer_names_list = image.layers
	
    visi_loop, other_layers, first_letter_layer, has_a_space = 0, 0, -1, False
    bumpmap_layer = "none"

    if use_texture:
        while visi_loop < num_layers:
            # check for layers to create bumpmap from
            visi_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            visi_name = visi_name.decode('utf-8')
            if visi_name == "space.png" or visi_name == " .png": has_a_space = True
            if len(visi_name) == 1 or visi_name == "space.png" or len(visi_name) == 5 and ".png" in visi_name:
                if first_letter_layer < 0: first_letter_layer = visi_loop
            else:
                pdb.gimp_image_select_item(image, 2, layer_names_list[visi_loop])
                is_empty = pdb.gimp_selection_is_empty(image)
                if is_empty == False: 
                    other_layers += 1
                    if other_layers == 1: 
                        # Use the top-most non-character layer as the source for the bumpmap layer and make a copy of it
                        bumpmap_layer = layer_names_list[visi_loop].copy()
                        image.add_layer(bumpmap_layer, -1)
                        bumpmap_layer_name = "Bump"
                        pdb.gimp_image_set_active_layer(image, bumpmap_layer)
                        pdb.gimp_selection_none(image)
                        # desaturate the newly created bumpmap layer
                        pdb.gimp_drawable_desaturate( bumpmap_layer, 3)
                    		
				
            visi_loop += 1
        # Update the lists of layers
        loop_ptr = 1
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
        loop = num_layers - loop_ptr
        layer_names_list = image.layers
	
    if other_layers == 0: use_texture = False # if there are no layers to use for the bumpmap then cancel it
	
    first_char_layer_name = "none"
	
    if not (use_sheen==False and use_shine == False):
        # turn off visibility of layers
        visi_loop, first_char_layer  = 0, -1
        while visi_loop < num_layers:
	        # get info about layer we are working on
            layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
            layer_name = layer_name.decode('utf-8')
            if len(layer_name) == 1 or len(layer_name) == 5 and ".png" in layer_name:
                if first_char_layer < 0: 
                    first_char_layer = visi_loop
                    first_char_layer_name = layer_name
            pdb.gimp_item_set_visible(layer_names_list[visi_loop],False)
            visi_loop += 1
			
    while loop >= 0:
	    # get info about layer we are working on
        layer_id = layer_ids[loop]
        layer_name = layer_names_list[loop]
	
        # cater for unicode characters above normal range
        display_layer_name =  pdb.gimp_item_get_name(layer_names_list[loop])
        display_layer_name = display_layer_name.decode('utf-8')	
	
        # check we have the layer id of the currently active layer
        pdb.gimp_image_set_active_layer(image, layer_name)
        layer_id = pdb.gimp_image_get_active_layer(image)

        # check to see if the current layer is a custom font character layer
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_id, display_layer_name)
		
        # if the outline width has been set to zero but no chisel or shine has been set don't do it
        #if use_chisel == False and use_shine == False: test_for_custom_layer = False
		
        if test_for_custom_layer:

            # ensure we have the active layer and make it visible
            active_layer_id = pdb.gimp_image_get_active_layer(image)
            pdb.gimp_item_set_visible(active_layer_id, True)

            if use_sheen:
                active_layer = image.active_layer 
                active_layer_id = proc_sheen(image, active_layer, black, white, effect_azimuth, gloss_colour, trim_value, shine_offset, polish_colour, effect_mode_ptr, add_sheen, emboss_width, sheen_opacity, highlight_blur)	
				
            if use_shine:
                active_layer = image.active_layer 
                active_layer_id = proc_gloss(image, active_layer, black, white, effect_azimuth, gloss_colour, trim_value, shine_offset, polish_colour, effect_mode_ptr, add_sheen, emboss_width, sheen_opacity, grey, highlight_blur)
  
						
				              
				
            if (num_layers - loop_ptr) > 0: pdb.gimp_item_set_visible(active_layer_id, False)
			
            num_layers, layer_ids = pdb.gimp_image_get_layers(image)
			
        loop_ptr += 1
        loop = num_layers - loop_ptr
        

    if first_char_layer_name <> "none":
        first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
        pdb.gimp_item_set_visible(first_char_layer_id, True)
        pdb.gimp_image_set_active_layer(image,first_char_layer_id)
	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
		
#### End of do_stuff

######################################################################################################################################

def proc_do_bump(image, other_layers, bumpmap_layer_id, bumpmap_layer_name, active_layer, black, white, bumpmap_width, waterlevel, ambient, effect_elevation, effect_azimuth, effect_mode_ptr):
    if other_layers > 0:
        work_layer = active_layer
        work_layer_name = work_layer.name
        width = work_layer.width
        height = work_layer.height
        bumpmap_width = bumpmap_width * 0.65
        if bumpmap_width < 1: bumpmap_width = 1
		
        if waterlevel > 0:
            waterlevel = waterlevel/100
            waterlevel = 1.0 - waterlevel
        else:
            waterlevel = 0.0
			
        if ambient > 0:
            ambient = ambient/100
            ambient = 1.0 - ambient
        else:
            ambient = 0.0        

        pdb.plug_in_bump_map(image, work_layer, bumpmap_layer_id, effect_azimuth, effect_elevation, bumpmap_width, 0, 0, waterlevel, ambient, True, False, 0)
		
        active_layer = image.active_layer 
        return active_layer
        
######################################################################################################################################
	
def proc_emboss(image, active_layer, black, white, effect_width, effect_elevation, effect_azimuth, emboss_blur, effect_mode_ptr, blur_radius, blur_delta):
    if effect_width < 1: effect_width = 1
    grey = gimpcolor.RGB(100, 100, 100)
    work_layer = active_layer # ie character layer
    work_layer_name = work_layer.name # ie character layer id
    width = work_layer.width
    height = work_layer.height
    colour = gimp.get_background()
    emboss_mode = fn_get_mode(effect_mode_ptr)
	
    layer_blur = image.height * 0.015
    if layer_blur < 1: layer_blur = 1	
	
    pdb.gimp_context_set_foreground(black)
    pdb.gimp_context_set_background(white)
	
    pdb.gimp_image_select_item(image, 2, work_layer)
	
    emboss_layer = gimp.Layer(image, "Emboss Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the bump map
    image.add_layer(emboss_layer, -1)
    emboss_layer.fill(1)
    pdb.gimp_drawable_edit_fill(emboss_layer, 0)
	
    #grey_layer = gimp.Layer(image, "Grey Layer", width, height, RGBA_IMAGE, 100, 0) # layer to become the bump map
    #image.add_layer(grey_layer, -1)
    #pdb.gimp_context_set_foreground(grey)	
    #grey_layer.fill(1)
    #pdb.gimp_drawable_edit_fill(grey_layer, 0)
	
    pdb.gimp_context_set_foreground(black)
    pdb.gimp_context_set_background(white)
	
    pdb.gimp_selection_none(image) ####
    pdb.plug_in_gauss_iir(image, emboss_layer, emboss_blur, True, True) # blur the bump map
	
    other_layers = 1
    bumpmap_layer_name = "Emboss Layer"
    bumpmap_layer = emboss_layer
	
    #active_layer = new_layer
	
    #temp values
    bump_waterlevel = 0
    bump_ambient = 0
    waterlevel = 0
    ambient = 0
    #end temp values
	
    bump_depth = effect_width
	
    work_layer = active_layer
    work_layer_name = work_layer.name
    width = work_layer.width
    height = work_layer.height
    #bumpmap_width = bumpmap_width * 0.65
    #if bumpmap_width < 1: bumpmap_width = 1
		
    if waterlevel > 0:
        waterlevel = waterlevel/100
        waterlevel = 1.0 - waterlevel
    else:
        waterlevel = 0.0
			
    if ambient > 0:
        ambient = ambient/100
        ambient = 1.0 - ambient
    else:
        ambient = 0.0        

    pdb.plug_in_bump_map(image, work_layer, emboss_layer, effect_azimuth, effect_elevation, bump_depth, 0, 0, waterlevel, ambient, True, False, 0)
    #work_layer = image.active_layer
	
    image.remove_layer(emboss_layer)
    #image.lower_layer(grey_layer)
    #temp values
    #blur_radius = 10
    #blur_delta = 8
    #end temp values
    pdb.gimp_image_select_item(image, 2, work_layer)
    ##pdb.plug_in_sel_gauss(image, work_layer, blur_radius, blur_delta)
    #pdb.plug_in_gauss_iir(image, grey_layer, 5, True, True) # blur the bump map
    #pdb.plug_in_gauss_iir2(image, grey_layer, blur_radius, blur_radius) # blur the bump map
    #pdb.plug_in_gauss_rle(image, grey_layer, blur_radius, True, True) # blur the bump map
    #work_layer.mode = emboss_mode
    #pdb.gimp_image_merge_down(image, work_layer, 0)
    work_layer = image.active_layer
    #pdb.gimp_item_set_name(work_layer, work_layer_name)
    #image.remove_layer(grey_layer)
		
    pdb.gimp_selection_none(image)
    active_layer = image.active_layer 
    ##pdb.plug_in_sel_gauss(image, active_layer, 10, 255)
    pdb.plug_in_sel_gauss(image, active_layer, blur_radius, blur_delta)
    return active_layer	
	
		
######################################################################################################################################

def proc_sheen(image, active_layer, black, white, effect_azimuth, gloss_colour, trim_value, shine_offset, polish_colour, effect_mode_ptr, add_sheen, emboss_width, sheen_opacity, highlight_blur):
    work_layer = active_layer
    work_layer_name = work_layer.name
    width = work_layer.width
    height = work_layer.height
    elevation = 85
    #emboss_width = 10
    colour = gimp.get_background()
    polish_mode = fn_get_mode(effect_mode_ptr)
    rotate = effect_azimuth
    effect_azimuth = 360-(effect_azimuth+90)
    if effect_azimuth>360: effect_azimuth = 360
	
    layer_blur = image.height * 0.015
    if layer_blur < 1: layer_blur = 1	
	
    sheen_size = int(max(width,height)*1.5)
    sheen_layer = gimp.Layer(image, "Sheen Layer", sheen_size, sheen_size, RGBA_IMAGE, 100, 0)
    image.add_layer(sheen_layer, -1)
    pdb.gimp_layer_set_offsets(sheen_layer, 0-int((sheen_size/2)-(width/2)), 0-int((sheen_size/2)-(height/2)))
    pdb.gimp_context_set_foreground(polish_colour)
    pdb.gimp_context_set_gradient_fg_transparent()
    pdb.gimp_drawable_edit_gradient_fill(sheen_layer, 0, 0, False, 1, 0, True, 0, 0, 0, int(sheen_size/2))
    pdb.gimp_item_transform_rotate(sheen_layer,math.radians(rotate),True, 0, 0)
    pdb.gimp_image_select_item(image, 2, work_layer)
    pdb.gimp_selection_shrink(image, 1)
    pdb.gimp_selection_invert(image)
    pdb.gimp_edit_clear(sheen_layer)
    pdb.gimp_selection_none(image)
    sheen_layer.mode = polish_mode
    pdb.gimp_layer_set_opacity(sheen_layer, sheen_opacity)
    work_layer = pdb.gimp_image_merge_down(image, sheen_layer, 2)
	
    new_layer = image.active_layer
    proc_trim_away(image, new_layer, trim_value)# shrink selection boundary by set amount of pixels
    active_layer = image.active_layer 
    return active_layer

    #### End of proc_sheen
	
######################################################################################################################################

def proc_gloss(image, active_layer, black, white, effect_azimuth, gloss_colour, trim_value, shine_offset, polish_colour, effect_mode_ptr, add_sheen, emboss_width, sheen_opacity, grey, highlight_blur):
    work_layer = active_layer
    work_layer_name = work_layer.name
    width = work_layer.width
    height = work_layer.height
    elevation = 85 # required to create pure white highlight
    user_emboss = emboss_width # Store user emboss width
    colour = gimp.get_background()
    polish_mode = fn_get_mode(effect_mode_ptr)
    rotate = effect_azimuth
    effect_azimuth = 360-(effect_azimuth+90)
    if effect_azimuth>360: effect_azimuth = 360
	
    #layer_blur = image.height * 0.015
    layer_blur = image.height * highlight_blur
    if layer_blur < 1: layer_blur = 1	
	
    if add_sheen:
        sheen_size = int(max(width,height)*1.5)
        sheen_layer = gimp.Layer(image, "Sheen Layer", sheen_size, sheen_size, RGBA_IMAGE, 100, 0)
        image.add_layer(sheen_layer, -1)
        pdb.gimp_layer_set_offsets(sheen_layer, 0-int((sheen_size/2)-(width/2)), 0-int((sheen_size/2)-(height/2)))
        pdb.gimp_context_set_foreground(polish_colour)
        pdb.gimp_context_set_gradient_fg_transparent()
        pdb.gimp_drawable_edit_gradient_fill(sheen_layer, 0, 0, False, 1, 0, True, 0, 0, 0, int(sheen_size/2))
        pdb.gimp_item_transform_rotate(sheen_layer,math.radians(rotate),True, 0, 0)
        pdb.gimp_image_select_item(image, 2, work_layer)
        pdb.gimp_selection_shrink(image, 1)
        pdb.gimp_selection_invert(image)
        pdb.gimp_edit_clear(sheen_layer)
        pdb.gimp_selection_none(image)
        sheen_layer.mode = polish_mode
        pdb.gimp_layer_set_opacity(sheen_layer, sheen_opacity)
        work_layer = pdb.gimp_image_merge_down(image, sheen_layer, 2)
		
    #pdb.gimp_context_set_foreground(black)
    pdb.gimp_context_set_foreground(grey)
    pdb.gimp_context_set_background(white)		
		
    pdb.gimp_image_select_item(image, 2, work_layer)
    new_layer = gimp.Layer(image, "Shine Layer", width, height, RGBA_IMAGE, 100, 0)
    image.add_layer(new_layer, -1)
    new_layer.fill(1) # flood layer with white
    pdb.gimp_drawable_edit_fill(new_layer, 0) # fill selection with shade of grey

    pdb.gimp_image_set_active_layer(image, new_layer)
    pdb.gimp_item_set_name(work_layer,"Original layer")
	
    # apply shine
    #pdb.gimp_context_set_foreground(black)
    #pdb.gimp_context_set_foreground(grey)
    #pdb.gimp_context_set_background(white)
	
    pdb.plug_in_gauss_rle2(image, new_layer, layer_blur, layer_blur) # apply blur filter to shine layer
    pdb.plug_in_emboss(image, new_layer, effect_azimuth, elevation, 5, 1) # apply emboss filter to shine layer
    pdb.gimp_selection_invert(image) # invert selection and
    pdb.gimp_drawable_edit_clear(new_layer) # clear background area of current character
    pdb.gimp_selection_invert(image) # reselect character area
    pdb.plug_in_colortoalpha(image ,new_layer ,(254, 254, 254)) # apply color to alpha to leave only white highlight
    pdb.gimp_context_set_sample_threshold_int(1)
    pdb.gimp_image_select_color(image, 2, new_layer, white) # select the white highlight area in case user wishes to replace this with another colour
    success = pdb.gimp_edit_cut(new_layer)
    pasted_layer = gimp.Layer(image, "Pasted Layer", width, height, RGBA_IMAGE, 100, 0)
    image.add_layer(pasted_layer, -1)
    floating_sel = pdb.gimp_edit_paste(pasted_layer,True)
    pdb.gimp_floating_sel_anchor(floating_sel)
    pdb.gimp_selection_none(image)
    pdb.gimp_drawable_offset(pasted_layer, False, 1, shine_offset, shine_offset)
    
    if gloss_colour:
        pdb.gimp_image_select_item(image, 2, pasted_layer)
        sel_is_empty = pdb.gimp_selection_is_empty(image) # better check that the selection still exists before shrinking
        if not sel_is_empty:
            pdb.gimp_image_select_item(image, 2, pasted_layer)
            pdb.gimp_context_set_background(colour) # get bg colour
            colour_layer = gimp.Layer(image, "Colour Layer", width, height, RGBA_IMAGE, 100, 35) # add a darken only mode layer
            pdb.gimp_selection_shrink(image, 1) # shrink selection boundary by 1 pixel
            image.add_layer(colour_layer, -1)
            colour_layer.fill(1) # fill the layer selection
            pdb.gimp_selection_none(image)
            pdb.gimp_image_merge_down(image, colour_layer, 0)
            pasted_layer = image.active_layer # reset layer id
	
    pdb.plug_in_blur(image, pasted_layer)
    #pdb.gimp_drawable_offset(new_layer, False, 1, shine_offset, shine_offset)
	
    #### New lines below
	    
    pdb.gimp_context_set_foreground(grey)
    pdb.gimp_context_set_background(white)
	
    pdb.gimp_image_select_item(image, 2, work_layer)
    new_layer_2 = gimp.Layer(image, "Shine Layer 2", width, height, RGBA_IMAGE, 100, 0)
    image.add_layer(new_layer_2, -1)
    new_layer_2.fill(1) # flood layer with white
    pdb.gimp_drawable_edit_fill(new_layer_2, 0) # fill selection with shade of grey
	
    pdb.gimp_image_set_active_layer(image, new_layer_2)
	
    pdb.plug_in_gauss_rle2(image, new_layer_2, layer_blur, layer_blur) # apply blur filter to shine layer
    pdb.plug_in_emboss(image, new_layer_2, effect_azimuth, elevation, emboss_width, 1) # apply emboss filter to shine layer
    pdb.gimp_selection_invert(image) # invert selection and
    pdb.gimp_drawable_edit_clear(new_layer_2) # clear background area of current character
    pdb.gimp_selection_invert(image) # reselect character area
	
    pdb.plug_in_colortoalpha(image ,new_layer_2 ,(255, 255, 255)) # apply color to alpha to leave only white highlight
	
    image.remove_layer(new_layer)
	
	
    new_layer_2 = pdb.gimp_image_merge_down(image, new_layer_2, 0)
    new_layer_2 = pdb.gimp_image_merge_down(image, new_layer_2, 0)
	
	
    #pdb.gimp_image_merge_down(image, pasted_layer, 0)
    new_layer = image.active_layer 
    pdb.gimp_item_set_name(new_layer, work_layer_name)
		
    pdb.gimp_selection_none(image)
    new_layer = image.active_layer
    proc_trim_away(image, new_layer, trim_value)# shrink selection boundary by set amount of pixels
    active_layer = image.active_layer 
    return active_layer

    #### End of proc_gloss
	
######################################################################################################################################

def proc_text_bevel(image, text_layer, bevel_width, bevel_depth, bevel_waterlevel, bevel_ambient, feather, black, white, midgrey, bevel_mode, bevel_azimuth, bevel_elevation, blur_radius, blur_delta):
    #### Create a bevel around text and return the new layer as bevel_layer
    #Rem Adjust values taken from dialog
    bevel_width = int(bevel_width * 2.55)
    if bevel_width < 2: bevel_width = 2
    blur_radius = blur_radius * 1
    if blur_delta>255: blur_delta = 255
	
    if bevel_ambient > 0.0:
        ambient = bevel_ambient/100
    else:
        ambient = 0.0
    if bevel_waterlevel > 0.0:
        waterlevel = bevel_waterlevel/100
    else:
        waterlevel = 0.0
		
    #Rem Select text outline and save it
    pdb.gimp_image_select_item(image, 2, text_layer)
    saved_selection = pdb.gimp_selection_save(image)

    #### Rem Make a bumpmap layer ######
    bumpmap_layer = text_layer.copy()
    image.add_layer(bumpmap_layer, -1)
    bumpmap_layer.name = text_layer.name + " Bumpmap"
    pdb.gimp_image_raise_item_to_top(image, bumpmap_layer)

    pdb.gimp_selection_feather(image, feather)
    blurred_selection = pdb.gimp_selection_save(image)
    loop = bevel_width
    pdb.gimp_selection_shrink(image, loop)

    while (pdb.gimp_selection_is_empty(image) == True ):
        loop = loop - 1
        pdb.gimp_image_select_item(image, 2, blurred_selection)
        pdb.gimp_selection_shrink(image, loop)
        
    bevel_width = min(bevel_width, loop)
    pdb.gimp_image_select_item(image, 2, blurred_selection)

    gimp.set_foreground(0, 0, 0)
    bumpmap_layer.fill(0)
	

    loop = 1
    #Rem Temp dirty fill_value 
    fill_value = (255 / bevel_width)
    while (loop <= bevel_width):

        #Rem Using Temp fill_value until something better comes to mind
        pdb.gimp_image_select_item(image, 2, blurred_selection)
        pdb.gimp_selection_shrink(image, loop -1)
        greycolour = int(fill_value)
        gimp.set_foreground(greycolour, greycolour, greycolour)

        pdb.gimp_edit_fill(bumpmap_layer, 0)

        #Rem Temp fill_value
        fill_value = fill_value + (255 / bevel_width)
        loop = loop + 1

    gimp.set_foreground(255, 255, 255)
    pdb.gimp_image_select_item(image, 2, blurred_selection)
    pdb.gimp_selection_shrink(image, bevel_width)

    if (pdb.gimp_selection_is_empty(image) == FALSE):
        pdb.gimp_edit_fill(bumpmap_layer, 0)

    pdb.gimp_selection_none(image)

    #### Make Bevel from bumpmap ####
    bevel_layer = text_layer.copy()
    image.add_layer(bevel_layer, -1)
    bevel_layer.name = text_layer.name + " Bevel"
    pdb.gimp_image_raise_item_to_top(image, bevel_layer)

    gimp.set_foreground(128, 128, 128)
    bevel_layer.fill(0)
    bevel_azimuth = bevel_azimuth - 180
    if bevel_azimuth < 0: bevel_azimuth = 360 + (bevel_azimuth)
    pdb.plug_in_bump_map(image, bevel_layer, bumpmap_layer, bevel_azimuth, bevel_elevation, bevel_depth, 0, 0, waterlevel, ambient, True, False, 0)
    bevel_layer.mode = bevel_mode
    pdb.gimp_layer_set_opacity(bevel_layer, 100)
	
    #### Delete area outside the bevel ####
    pdb.gimp_image_select_item(image, 2, saved_selection)
    pdb.gimp_selection_invert(image)

    if (pdb.gimp_selection_is_empty(image) == False):
        pdb.gimp_edit_clear(bevel_layer)

    pdb.gimp_image_select_item(image, 2, saved_selection)
    pdb.gimp_selection_shrink(image, bevel_width)

    if (pdb.gimp_selection_is_empty(image) == False):
        pdb.gimp_edit_clear(bevel_layer)
	
    pdb.gimp_image_select_item(image, 2, saved_selection)
 
    pdb.gimp_selection_none(image)
    pdb.gimp_drawable_set_visible(bumpmap_layer, False)	
    image.remove_channel(saved_selection)
    image.remove_channel(blurred_selection)
    image.remove_layer(bumpmap_layer)
    pdb.gimp_image_merge_down(image, bevel_layer, 0)
    active_layer = image.active_layer 
    pdb.plug_in_sel_gauss(image, active_layer, blur_radius, blur_delta)
    active_layer = image.active_layer 
    return active_layer
	
#### End of proc_text_bevel

######################################################################################################################################
	
def proc_trim_away(img, layer, amount):    
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    
######################################################################################################################################

def fn_test_for_custom_layer(img,l_id,l_name):
    # checks that the current layer is compatible with custom fonts
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5 or len(l_name) == 6:
        
        if ".png" in l_name:
            test = True
            if len(l_name) == 6:
                part_name_val = unicode(l_name[0:2])
                if part_name_val <= 65535: test = False
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################

def fn_get_mode(new_mode):
    temp_mode = 0
    n_mode_info = "Normal"
    if new_mode <> 0:
        if new_mode == 1:
            temp_mode = 1
            n_mode_info = "Dissolve"
        elif new_mode == 2:
            temp_mode = 57
            n_mode_info = "Color Erase"
        elif new_mode == 3:
            temp_mode = 59
            n_mode_info = "Merge"
        elif new_mode == 4:
            temp_mode = 60
            n_mode_info = "Split"
        elif new_mode == 5:
            temp_mode = 36
            n_mode_info = "Lighten Only"
        elif new_mode == 6:
            temp_mode = 55
            n_mode_info = "Luma Luminance Lighten"
        elif new_mode == 7:
            temp_mode = 31
            n_mode_info = "Screen"
        elif new_mode == 8:
            temp_mode = 42
            n_mode_info = "Dodge"
        elif new_mode == 9:
            temp_mode = 33
            n_mode_info = "Addition"
        elif new_mode == 10:
            temp_mode = 35
            n_mode_info = "Darken only"
        elif new_mode == 11:
            temp_mode = 54
            n_mode_info = "Luma Luminance darken"
        elif new_mode == 12:
            temp_mode = 30
            n_mode_info = "Multiply"
        elif new_mode == 13:
            temp_mode = 43
            n_mode_info = "Burn"
        elif new_mode == 14:
            temp_mode = 53
            n_mode_info = "Linear burn"
        elif new_mode == 15:
            temp_mode = 23
            n_mode_info = "Overlay"
        elif new_mode == 16:
            temp_mode = 45
            n_mode_info = "Soft light"
        elif new_mode == 17:
            temp_mode = 44
            n_mode_info = "Hard light"
        elif new_mode == 18:
            temp_mode = 48
            n_mode_info = "Vivid light"
        elif new_mode == 19:
            temp_mode = 49
            n_mode_info = "Pin light"
        elif new_mode == 20:
            temp_mode = 50
            n_mode_info = "Linear light"
        elif new_mode == 21:
            temp_mode = 51
            n_mode_info = "Hard mix"
        elif new_mode == 22:
            temp_mode = 32
            n_mode_info = "Difference"
        elif new_mode == 23:
            temp_mode = 52
            n_mode_info = "Exclusion"
        elif new_mode == 24:
            temp_mode = 34
            n_mode_info = "Subtract"
        elif new_mode == 25:
            temp_mode = 46
            n_mode_info = "Grain extract"
        elif new_mode == 26:
            temp_mode = 47
            n_mode_info = "Grain merge"
        elif new_mode == 27:
            temp_mode = 41
            n_mode_info = "Divide"
        elif new_mode == 28:
            temp_mode = 37
            n_mode_info = "Hue HSV"
        elif new_mode == 29:
            temp_mode = 38
            n_mode_info = "Saturation HSV"
        elif new_mode == 30:
            temp_mode = 39
            n_mode_info = "Color HSL"
        elif new_mode == 31:
            temp_mode = 40
            n_mode_info = "Value HSL"
        elif new_mode == 32:
            temp_mode = 24
            n_mode_info = "Hue LCH"
        elif new_mode == 33:
            temp_mode = 25
            n_mode_info = "Chroma LCH"
        elif new_mode == 34:
            temp_mode = 26
            n_mode_info = "Color LCH"
        elif new_mode == 35:
            temp_mode = 27
            n_mode_info = "Lightness LCH"
        elif new_mode == 36:
            temp_mode = 56
        n_mode_info = "Luminance"
    mode = temp_mode

    return mode
	
######################################################################################################################################

def fn_set_bevel_mode(bevel_mode_ptr):
    # Use New Modes when creating a bevel
    bevel_mode = 23
    if bevel_mode_ptr == 0:
        bevel_mode = 23 #Overlay Mode
    elif bevel_mode_ptr == 1:
        bevel_mode = 30 #Multiply Mode
    elif bevel_mode_ptr == 2:
        bevel_mode = 35 #Darken Only Mode
    elif bevel_mode_ptr == 3:
        bevel_mode = 40 #HSV Value Mode
    elif bevel_mode_ptr == 4:
        bevel_mode = 41 #Divide Mode
    elif bevel_mode_ptr == 5:
        bevel_mode = 42 #Dodge Mode
    elif bevel_mode_ptr == 6:
        bevel_mode = 44 # Hardlight Mode
    if bevel_mode_ptr == 7:
        bevel_mode = 45 # Softlight Mode
    elif bevel_mode_ptr == 8:
        bevel_mode = 46 #Grain Extract Mode
    elif bevel_mode_ptr == 9:
        bevel_mode = 47 #Grain Merge Mode
    elif bevel_mode_ptr == 10:
        bevel_mode = 48 #Vivid Mode
    elif bevel_mode_ptr == 11:
        bevel_mode = 50 #Linear Light Mode
    elif bevel_mode_ptr == 12:
        bevel_mode = 51 #Hard Mix Mode
    elif bevel_mode_ptr == 13:
        bevel_mode = 56 #Luminance Mode
    return bevel_mode


######################################################################################################################################

register(
    "python_fu_Custom_Font_Sheen_N_Shine_v0-04",
    "Adds shine effects to custom fonts",
    "Provides options for adding shine effects to custom fonts",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2020",
    "<Image>/Filters/Custom Font Tools/Custom Font Sheen N Shine v0.04...",
    "*",
    [
    (PF_TOGGLE, "pf_add_shine", "Add Shine effect: ", 1),
    (PF_TOGGLE, "pf_add_sheen_effect", "Add Sheen effect only: ", 0),
    (PF_SPINNER, "pf_emboss_width", "Emboss width ", 10,(1,50,1)),
    (PF_SPINNER, "pf_highlight_adj", "Highlight blur adjustment: ", 0.015,(0.005,0.1,0.005)),
    (PF_SPINNER, "pf_shine_offset", "Shine offset amount: ", 0,(0,20,1)),
    (PF_SPINNER, "pf_shine_trim", "Trim shine in pixels: ", 0,(0,10,1)),
    (PF_SLIDER, "pf_emboss_colour", "Emboss emphasis: ", 50, (0, 150, 1)),
    (PF_TOGGLE, "pf_shine_colour", "Use (BG) colour for Shine: ", 0),
    (PF_TOGGLE, "pf_add_sheen", "Add Sheen (BG colour) to Shine: ", 0),
    (PF_SLIDER, "pf_opacity", "Sheen opacity: ", 60, (0, 100, 5)),
    (PF_SPINNER, "pf_blur_radius", "Artifact blur radius: ", 10,(0.0,100.0,0.1)),
    (PF_SLIDER, "pf_blur_delta", "Artifact blur delta amount: ", 15,(0,255,1)),
    (PF_SPINNER, "pf_effect_elevation", "Effect elevation: ", 45.0,(0.5,90,0.1)),
    (PF_SPINNER, "pf_shine_azimuth", "Effect azimuth: ", 315,(0,360,1)),
    (PF_OPTION, "mode_option2", "New Mode to use: ", 26, ("Normal", "Dissolve","Color erase", "Merge", "Split", "Lighten only", "Luma/Luminance lighten only", "Screen", "Dodge", "Addition", "Darken only", "Luma/Luminance darken only", "Multiply", "Burn", "Linear burn", "Overlay", "Soft light", "Hard light", "Vivid light", "Pin light", "Linear light", "Hard mix", "Difference", "Exclusion", "Subtract", "Grain extract", "Grain merge", "Divide", "Hue (HSV)", "Saturation (HSV)", "Color (HSL)", "Value (HSL)", "Hue (LCH)", "Chroma (LCH)", "Color (LCH)", "Lightness (LCH)", "Luminance")),
    ],
    [],
    do_stuff)

main()

	

